@@ -1,6 +1,6 @@
 ;; jabber-alert.el - alert hooks
 
-;; Copyright (C) 2003, 2004, 2005, 2007 - Magnus Henoch - mange@freemail.hu
+;; Copyright (C) 2003, 2004, 2005, 2007, 2008 - Magnus Henoch - mange@freemail.hu
 ;; Copyright (C) 2002, 2003, 2004 - tom berger - object@intelectronica.net
 
 ;; This file is a part of jabber.el.
@@ -20,6 +20,7 @@
 ;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 (require 'jabber-util)
+(require 'jabber-autoloads)
 
 (require 'cl)
 
@@ -400,6 +401,16 @@ This function is not called directly, but is the default for
 	     "")))
       (concat formattedname formattedstatus formattedtext)))))
 
+(defun jabber-presence-only-chat-open-message (who oldstatus newstatus statustext)
+  "This function returns the same as `jabber-presence-default-message' but only 
+if there is a chat buffer open for WHO, keeping the amount of presence messages 
+at a more manageable level when there are lots of users.
+
+This function is not called directly, but can be used as the value for
+`jabber-alert-presence-message-function'."
+  (when (get-buffer (jabber-chat-get-buffer (jabber-xml-get-attribute xml-data 'from)))
+    (jabber-presence-default-message who oldstatus newstatus statustext)))
+
 (defun jabber-presence-wave (who oldstatus newstatus statustext proposed-alert)
   "Play the wave file specified in `jabber-alert-presence-wave'"
   (when proposed-alert
@@ -470,6 +481,39 @@ This function uses `jabber-info-message-alist' to find a message."
 (define-personal-jabber-alert jabber-muc-switch)
 (define-personal-jabber-alert jabber-muc-display)
 
+(defcustom jabber-autoanswer-alist nil
+  "Specific phrases to autoanswer on specific message.
+The keys are regexps matching the incoming message text, and the values are
+autoanswer phrase."
+  :type '(alist :key-type regexp :value-type string)
+  :group 'jabber-alerts)
+
+(defun jabber-autoanswer-answer (from buffer text proposed-alert)
+  "Answer automaticaly when incoming text matches first element
+of `jabber-autoanswer-alist'"
+  (when (and proposed-alert jabber-autoanswer-alist)
+    (let ((message
+           (dolist (entry jabber-autoanswer-alist)
+             (when (string-match (car entry) text)
+               (return (cdr entry))))))
+      (if message
+          (jabber-chat-send jabber-buffer-connection message)))
+    ))
+(pushnew 'jabber-autoanswer-answer (get 'jabber-alert-message-hooks 'custom-options))
+
+(defun jabber-autoanswer-answer-muc (nick group buffer text proposed-alert)
+  "Answer automaticaly when incoming text matches first element
+of `jabber-autoanswer-alist'"
+  (when (and proposed-alert jabber-autoanswer-alist)
+    (let ((message
+           (dolist (entry jabber-autoanswer-alist)
+             (when (string-match (car entry) text)
+               (return (cdr entry))))))
+      (if message
+          (jabber-chat-send jabber-buffer-connection message)))
+    ))
+(pushnew 'jabber-autoanswer-answer-muc (get 'jabber-alert-muc-hooks 'custom-options))
+
 (provide 'jabber-alert)
 
 ;;; arch-tag: 725bd73e-c613-4fdc-a11d-3392a7598d4f
