@@ -22,9 +22,11 @@
 (require 'jabber-chat)
 (require 'jabber-widget)
 (require 'jabber-newdisco)
+(require 'jabber-autoloads)
 
 (require 'cl)
 
+;;;###autoload
 (defvar *jabber-active-groupchats* nil
   "alist of groupchats and nicknames
 Keys are strings, the bare JID of the room.
@@ -47,6 +49,16 @@ Values are lists of nickname strings.")
 (defvar jabber-muc-topic ""
   "The topic of the current MUC room.")
 
+(defvar jabber-role-history ()
+  "Keeps track of previously used roles")
+
+(defvar jabber-affiliation-history ()
+  "Keeps track of previously used affiliations")
+
+(defvar jabber-muc-nickname-history ()
+  "Keeps track of previously referred-to nicknames")
+
+;;;###autoload
 (defcustom jabber-muc-default-nicknames nil
   "Default nickname for specific MUC rooms."
   :group 'jabber-chat
@@ -55,6 +67,7 @@ Values are lists of nickname strings.")
 		(string :tag "JID of room")
 		(string :tag "Nickname"))))
 
+;;;###autoload
 (defcustom jabber-muc-autojoin nil
   "List of MUC rooms to automatically join on connection.
 This list is saved in your Emacs customizations.  You can also store
@@ -135,10 +148,12 @@ The format is that of `mode-line-format' and `header-line-format'."
   :type 'sexp
   :group 'jabber-chat)
 
+;;;###autoload
 (defvar jabber-muc-printers '()
   "List of functions that may be able to print part of a MUC message.
 This gets prepended to `jabber-chat-printers', which see.")
 
+;;;###autoload
 (defun jabber-muc-get-buffer (group)
   "Return the chat buffer for chatroom GROUP.
 Either a string or a buffer is returned, so use `get-buffer' or
@@ -163,6 +178,7 @@ This function is idempotent."
     (setq header-line-format jabber-muc-header-line-format)
     (current-buffer)))
 
+;;;###autoload
 (defun jabber-muc-private-get-buffer (group nickname)
   "Return the chat buffer for private chat with NICKNAME in GROUP.
 Either a string or a buffer is returned, so use `get-buffer' or
@@ -342,7 +358,7 @@ JID; only provide completion as a guide."
   (let ((nicknames (cdr (assoc group jabber-muc-participants))))
     (unless nicknames
       (error "Unknown group: %s" group))
-    (completing-read prompt nicknames nil t)))
+    (completing-read prompt nicknames nil t nil 'jabber-muc-nickname-history)))
 
 (add-to-list 'jabber-jid-muc-menu
    (cons "Configure groupchat" 'jabber-groupchat-get-config))
@@ -557,7 +573,7 @@ groupchat buffer."
    (jabber-muc-argument-list
     (let ((nickname (jabber-muc-read-nickname jabber-group "Nickname: ")))
       (list nickname
-	    (completing-read "New role: " '(("none") ("visitor") ("participant") ("moderator")) nil t)
+	    (completing-read "New role: " '(("none") ("visitor") ("participant") ("moderator")) nil t nil 'jabber-role-history)
 	    (read-string "Reason: ")))))
   (unless (or (zerop (length nickname)) (zerop (length role)))
     (jabber-send-iq jc group "set"
@@ -585,7 +601,7 @@ group, else it is a JID."
 	 (jabber-read-jid-completing "User: "))
        nickname-p
        (completing-read "New affiliation: "
-			'(("none") ("outcast") ("member") ("admin") ("owner")) nil t)
+			'(("none") ("outcast") ("member") ("admin") ("owner")) nil t nil 'jabber-affiliation-history)
        (read-string "Reason: ")))))
   (let ((jid
 	 (if nickname-p
@@ -649,7 +665,7 @@ group, else it is a JID."
 	      (let ((action
 		     `(lambda (&rest ignore) (interactive)
 			(jabber-groupchat-join jabber-buffer-connection ,group
-					       (jabber-muc-read-my-nickname ,jabber-buffer-connection ,group)))))
+					       (jabber-muc-read-my-nickname jabber-buffer-connection ,group)))))
 		(if (fboundp 'insert-button)
 		    (insert-button "Accept"
 				   'action action)
@@ -707,6 +723,7 @@ group, else it is a JID."
 				  (or (plist-get bookmark :nick)
 				      (plist-get (fsm-get-state-data jc) :username)))))))))
 
+;;;###autoload
 (defun jabber-muc-message-p (message)
   "Return non-nil if MESSAGE is a groupchat message.
 That does not include private messages in a groupchat, but does
@@ -722,11 +739,13 @@ include groupchat invites."
 	  (gethash (jabber-jid-symbol from) jabber-pending-groupchats))
      (jabber-xml-path message '(("http://jabber.org/protocol/muc#user" . "x") invite)))))
 
+;;;###autoload
 (defun jabber-muc-sender-p (jid)
   "Return non-nil if JID is a full JID of an MUC participant."
   (and (assoc (jabber-jid-user jid) *jabber-active-groupchats*)
        (jabber-jid-resource jid)))
 
+;;;###autoload
 (defun jabber-muc-private-message-p (message)
   "Return non-nil if MESSAGE is a private message in a groupchat."
   (let ((from (jabber-xml-get-attribute message 'from))
@@ -874,32 +893,45 @@ Return nil if X-MUC is nil."
 	 (item (car (jabber-xml-get-children x-muc 'item)))
 	 (actor (jabber-xml-get-attribute (car (jabber-xml-get-children item 'actor)) 'jid))
 	 (reason (car (jabber-xml-node-children (car (jabber-xml-get-children item 'reason)))))
-	 (status-code (jabber-xml-get-attribute
-		       (car (jabber-xml-get-children x-muc 'status))
-		       'code))
-	 (error-node (car (jabber-xml-get-children presence 'error))))
+	 (error-node (car (jabber-xml-get-children presence 'error)))
+	 (status-code (if error-node
+			  (jabber-xml-get-attribute error-node 'code)
+			(jabber-xml-get-attribute (car (jabber-xml-get-children x-muc 'status)) 'code))))
     ;; handle leaving a room
     (cond 
      ((or (string= type "unavailable") (string= type "error"))
       ;; error from room itself? or are we leaving?
       (if (or (null nickname)
 	      (string= nickname (gethash (jabber-jid-symbol group) jabber-pending-groupchats)))
-	  (let ((message (cond
-			  ((string= type "error")
-			   (concat "Error entering room"
-				   (when error-node
-				     (concat ": " (jabber-parse-error error-node)))))
-			  ((equal status-code "301")
-			   (concat "You have been banned"
-				   (when actor (concat " by " actor))
-				   (when reason (concat " - '" reason "'"))))
-			  ((equal status-code "307")
-			   (concat "You have been kicked"
-				   (when actor (concat " by " actor))
-				   (when reason (concat " - '" reason "'"))))
-			  (t
-			   "You have left the chatroom"))))
-	    (jabber-muc-remove-groupchat group)
+	  ;; Assume that an error means that we were thrown out of the
+	  ;; room...
+	  (let* ((leavingp t)
+		 (message (cond
+			   ((string= type "error")
+			    (cond
+			     ;; ...except for certain cases.
+			     ((or (equal status-code "406")
+				  (equal status-code "409"))
+			      (setq leavingp nil)
+			      (concat "Nickname change not allowed"
+				      (when error-node
+					(concat ": " (jabber-parse-error error-node)))))
+			     (t
+			      (concat "Error entering room"
+				      (when error-node
+					(concat ": " (jabber-parse-error error-node)))))))
+			   ((equal status-code "301")
+			    (concat "You have been banned"
+				    (when actor (concat " by " actor))
+				    (when reason (concat " - '" reason "'"))))
+			   ((equal status-code "307")
+			    (concat "You have been kicked"
+				    (when actor (concat " by " actor))
+				    (when reason (concat " - '" reason "'"))))
+			   (t
+			    "You have left the chatroom"))))
+	    (when leavingp
+	      (jabber-muc-remove-groupchat group))
 	    ;; If there is no buffer for this groupchat, don't bother
 	    ;; creating one just to tell that user left the room.
 	    (let ((buffer (get-buffer (jabber-muc-get-buffer group))))
