@@ -25,6 +25,7 @@
 (require 'jabber-util)
 (require 'jabber-menu)
 (require 'jabber-muc)
+(require 'jabber-autoloads)
 
 (require 'assoc)
 
@@ -34,6 +35,9 @@ Each function takes one argument, the connection, and returns a
 possibly empty list of extra child element of the <presence/>
 stanza.")
 
+(defvar jabber-presence-history ()
+  "Keeps track of previously used presence status types")
+
 (add-to-list 'jabber-iq-set-xmlns-alist
 	     (cons "jabber:iq:roster" (function (lambda (jc x) (jabber-process-roster jc x nil)))))
 (defun jabber-process-roster (jc xml-data closure-data)
@@ -285,6 +289,7 @@ CLOSURE-DATA should be 'initial if initial roster push, nil otherwise."
 	  (setq count (1+ count))))
     count))
 
+;;;###autoload
 (defun jabber-send-presence (show status priority)
   "Set presence for all accounts."
   (interactive (list (completing-read "show:"
@@ -293,7 +298,7 @@ CLOSURE-DATA should be 'initial if initial roster push, nil otherwise."
 					("xa" . nil)
 					("dnd" . nil)
 					("chat" . nil))
-				      nil t)
+				      nil t nil 'jabber-presence-history)
 		     (jabber-read-with-input-method "status message: " *jabber-current-status* '*jabber-status-history*)
 		     (read-string "priority: " (progn
 						 (unless *jabber-current-priority*
@@ -330,7 +335,8 @@ CLOSURE-DATA should be 'initial if initial roster push, nil otherwise."
        `(status () ,*jabber-current-status*))
     ,(when (> (length *jabber-current-show*) 0)
 	 `(show () ,*jabber-current-show*))
-    (priority () ,(number-to-string *jabber-current-priority*))
+    ,(when *jabber-current-priority*
+       `(priority () ,(number-to-string *jabber-current-priority*)))
     ,@(apply 'append (mapcar (lambda (f)
 			       (funcall f jc))
 			     jabber-presence-element-functions))))
@@ -369,7 +375,7 @@ TYPE is one of:
 			    ("unsubscribe")
 			    ("subscribed")
 			    ("unsubscribed"))
-			  nil t nil nil "online")))
+			  nil t nil 'jabber-presence-history "online")))
   (cond
    ((member type '("probe" "unavailable" 
 		   "subscribe" "unsubscribe"
@@ -392,16 +398,17 @@ With prefix argument, ask for status message."
   (interactive (list
 		(when current-prefix-arg
 		  (jabber-read-with-input-method "status message: " *jabber-current-status* '*jabber-status-history*))))
-  (jabber-send-presence "away" status *jabber-current-priority*))
+  (jabber-send-presence "away" (if status status *jabber-current-status*) *jabber-current-priority*))
 
 (defun jabber-send-xa-presence (&optional status)
   "Send extended away presence.
 With prefix argument, ask for status message."
   (interactive (list
-		(when current-prefix-arg
-		  (jabber-read-with-input-method "status message: " *jabber-current-status* '*jabber-status-history*))))
-  (jabber-send-presence "xa" status *jabber-current-priority*))
+        (when current-prefix-arg
+          (jabber-read-with-input-method "status message: " *jabber-current-status* '*jabber-status-history*))))
+  (jabber-send-presence "xa" (if status status *jabber-current-status*) *jabber-current-priority*))
 
+;;;###autoload
 (defun jabber-send-default-presence (&optional jc)
   "Send default presence.
 Default presence is specified by `jabber-default-priority', `jabber-default-show',
@@ -409,7 +416,10 @@ and `jabber-default-status'."
   (interactive)
   ;; jc is ignored.  It's only there so this function can be in
   ;; jabber-post-connect-hooks.
-  (jabber-send-presence jabber-default-show jabber-default-status jabber-default-priority))
+  (jabber-send-presence
+   jabber-default-show
+   (if (not (string= jabber-default-status "")) jabber-default-status *jabber-current-status*)
+   jabber-default-priority))
 
 (defun jabber-send-current-presence (&optional jc)
   "(Re-)send current presence.
@@ -436,17 +446,40 @@ else send defaults (see `jabber-send-default-presence')."
 		      ,@(when (and request (> (length request) 0))
 			  (list `(status () ,request))))))
 
+(defvar jabber-roster-group-history nil
+  "History of entered roster groups")
+
 (add-to-list 'jabber-jid-roster-menu
 	     (cons "Add/modify roster entry" 'jabber-roster-change))
 (defun jabber-roster-change (jc jid name groups)
   "Add or change a roster item."
   (interactive (let* ((jid (jabber-jid-symbol
 			    (jabber-read-jid-completing "Add/change JID: ")))
+		      (account (jabber-read-account))
 		      (name (get jid 'name))
-		      (groups (get jid 'groups)))
-		 (list (jabber-read-account)
+		      (groups (get jid 'groups))
+		      (all-groups
+		       (apply #'append
+			      (mapcar
+			       (lambda (j) (get j 'groups))
+			       (plist-get (fsm-get-state-data account) :roster)))))
+		 (when (string< emacs-version "22")
+		   ;; Older emacsen want the completion table to be an alist...
+		   (setq all-groups (mapcar #'list all-groups)))
+		 (list account
 		       jid (jabber-read-with-input-method (format "Name: (default `%s') " name) nil nil name)
-		       (car (read-from-string (jabber-read-with-input-method (format "Groups: (default `%S') " groups) nil nil (format "%S" groups)))))))
+		       (delete ""
+			       (completing-read-multiple 
+				(format
+				 "Groups, comma-separated: (default %s) "
+				 (if groups
+				     (mapconcat #'identity groups ",")
+				   "none"))
+				all-groups
+				nil nil nil
+				'jabber-roster-group-history
+				(mapconcat #'identity groups ",")
+				t)))))
   ;; If new fields are added to the roster XML structure in a future standard,
   ;; they will be clobbered by this function.
   ;; XXX: specify account
