@@ -1,6 +1,6 @@
 ;; jabber-roster.el - displaying the roster    -*- coding: utf-8; -*-
 
-;; Copyright (C) 2003, 2004, 2007 - Magnus Henoch - mange@freemail.hu
+;; Copyright (C) 2003, 2004, 2007, 2008 - Magnus Henoch - mange@freemail.hu
 ;; Copyright (C) 2002, 2003, 2004 - tom berger - object@intelectronica.net
 
 ;; This file is a part of jabber.el.
@@ -97,7 +97,8 @@ These functions should take two roster items A and B, and return:
 >0 if A > B"
   :type 'hook
   :options '(jabber-roster-sort-by-status 
-	     jabber-roster-sort-by-displayname)
+	     jabber-roster-sort-by-displayname
+	     jabber-roster-sort-by-group)
   :group 'jabber-roster)
 
 (defcustom jabber-sort-order '("chat" "" "away" "dnd" "xa")
@@ -107,7 +108,12 @@ Offline is represented as nil."
   :group 'jabber-roster)
 
 (defcustom jabber-show-resources 'sometimes
-  "Show resources in roster?"
+  "Show contacts' resources in roster?
+This can be one of the following symbols:
+
+nil       Never show resources
+sometimes Show resources when there are more than one
+always    Always show resources"
   :type '(radio (const :tag "Never" nil)
 		(const :tag "When more than one connected resource" sometimes)
 		(const :tag "Always" always))
@@ -176,6 +182,7 @@ Trailing newlines are always removed, regardless of this variable."
 
 (defvar jabber-roster-mode-map 
   (let ((map (make-sparse-keymap)))
+    (suppress-keymap map)
     (set-keymap-parent map jabber-common-keymap)
     (define-key map [mouse-2] 'jabber-popup-combined-menu)
     (define-key map (kbd "TAB") 'jabber-go-to-next-jid)
@@ -217,6 +224,7 @@ bring up menus of actions.
 
 (put 'jabber-roster-mode 'mode-class 'special)
 
+;;;###autoload
 (defun jabber-switch-to-roster-buffer (&optional jc)
   "Switch to roster buffer.
 Optional JC argument is ignored; it's there so this function can
@@ -390,7 +398,11 @@ H        Toggle displaying this text
 	  (dolist (hook '(jabber-info-message-hooks jabber-alert-info-message-hooks))
 	    (run-hook-with-args hook 'roster (current-buffer) (funcall jabber-alert-info-message-function 'roster (current-buffer)))))
       (when current-line
-	(goto-line current-line)
+	;; Go back to previous line - don't use goto-line, since it
+	;; sets the mark.
+	(goto-char (point-min))
+	(forward-line (1- current-line))
+	;; ...and go back to previous column
 	(move-to-column current-column)))))
 
 (defun jabber-display-roster-entry (jc buddy)
@@ -475,6 +487,7 @@ BUDDY is a JID symbol."
 				 resource-str)
 	    (insert "\n" resource-str)))))))
 
+;;;###autoload
 (defun jabber-roster-update (jc new-items changed-items deleted-items)
   "Update roster, in memory and on display.
 Add NEW-ITEMS, update CHANGED-ITEMS and remove DELETED-ITEMS, all
