@@ -180,8 +180,12 @@ Return nil if none found."
     (intern (downcase (jabber-jid-user string)) jabber-jid-obarray)))
 
 (defun jabber-my-jid-p (jc jid)
-  "Return non-nil if the specified JID is in jabber-account-list (modulo resource). JC arg placed for compatibility and may be removed in future."
-  (member (jabber-jid-user jid) (mapcar (lambda (x) (jabber-jid-user (car x))) jabber-account-list)))
+  "Return non-nil if the specified JID is in jabber-account-list (modulo resource).
+Also return non-nil if JID matches JC, modulo resource."
+  (or
+   (equal (jabber-jid-user jid)
+	  (jabber-connection-bare-jid jc))
+   (member (jabber-jid-user jid) (mapcar (lambda (x) (jabber-jid-user (car x))) jabber-account-list))))
 
 (defun jabber-read-jid-completing (prompt &optional subset require-match default resource)
   "read a jid out of the current roster from the minibuffer.
@@ -278,7 +282,7 @@ bare-or-muc Turn full JIDs to bare ones, except for in MUC"
   "Uncache cached password for BARE-JID.
 Useful if the password proved to be wrong."
   (interactive (list (jabber-jid-user
-		      (completing-read "Forget password of account: " jabber-account-list))))
+		      (completing-read "Forget password of account: " jabber-account-list nil nil nil 'jabber-account-history))))
   (when (fboundp 'password-cache-remove)
     (password-cache-remove (jabber-password-key bare-jid))))
 
@@ -318,7 +322,7 @@ account."
                       (concat "Select Jabber account (default "
                               default
                               "): ")
-                      completions nil t nil nil
+                      completions nil t nil 'jabber-account-history
                       default)))
          (cdr (assoc input completions))))))))
 
@@ -408,7 +412,9 @@ TIME is in a format accepted by `format-time-string'."
 
 (defun jabber-report-success (jc xml-data context)
   "IQ callback reporting success or failure of the operation.
-CONTEXT is a string describing the action."
+CONTEXT is a string describing the action.
+\"CONTEXT succeeded\" or \"CONTEXT failed: REASON\" is displayed in
+the echo area."
   (let ((type (jabber-xml-get-attribute xml-data 'type)))
     (message (concat context
 		     (if (string= type "result")
@@ -609,8 +615,9 @@ See Info node `(jabber)XMPP URIs'."
     (cond
      ;; Join an MUC.
      ((string= method "join")
-      (jabber-groupchat-join
-       (jabber-read-account) jid (jabber-muc-read-my-nickname jid) t))
+      (let ((account (jabber-read-account)))
+	(jabber-groupchat-join
+	 account jid (jabber-muc-read-my-nickname account jid) t)))
      ;; Register with a service.
      ((string= method "register")
       (jabber-get-register (jabber-read-account) jid))
